# app.py
from flask import Flask, render_template, request, jsonify
import time
import json
import os
import random
import string
from datetime import datetime
from exploit_modules.cve_2025_30065 import CVE_2025_30065

app = Flask(__name__)

# Create necessary directories
os.makedirs('exploit_modules', exist_ok=True)
os.makedirs('static', exist_ok=True)
os.makedirs('templates', exist_ok=True)


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/api/exploit/init', methods=['POST'])
def initialize_exploit():
    data = request.json
    target_ip = data.get('target_ip', '192.168.1.100')
    target_port = data.get('target_port', '8080')

    # Initialize the CVE exploit module
    cve = CVE_2025_30065(target_ip, target_port)

    return jsonify({
        'status': 'initialized',
        'cve_id': cve.CVE_ID,
        'target': f"{target_ip}:{target_port}",
        'timestamp': datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    })


@app.route('/api/exploit/recon', methods=['POST'])
def run_recon():
    data = request.json
    target_ip = data.get('target_ip', '192.168.1.100')

    cve = CVE_2025_30065(target_ip, '8080')
    result = cve.run_recon()

    # Simulate processing time
    time.sleep(1)

    return jsonify(result)


@app.route('/api/exploit/exploit', methods=['POST'])
def run_exploit():
    data = request.json
    target_ip = data.get('target_ip', '192.168.1.100')

    cve = CVE_2025_30065(target_ip, '8080')
    result = cve.run_exploit()

    # Simulate processing time
    time.sleep(2)

    return jsonify(result)


@app.route('/api/exploit/privilege', methods=['POST'])
def run_privilege_escalation():
    data = request.json
    target_ip = data.get('target_ip', '192.168.1.100')

    cve = CVE_2025_30065(target_ip, '8080')
    result = cve.run_privilege_escalation()

    # Simulate processing time
    time.sleep(1.5)

    return jsonify(result)


@app.route('/api/exploit/exfiltration', methods=['POST'])
def run_exfiltration():
    data = request.json
    target_ip = data.get('target_ip', '192.168.1.100')

    cve = CVE_2025_30065(target_ip, '8080')
    result = cve.run_exfiltration()

    # Simulate processing time
    time.sleep(2.5)

    return jsonify(result)


@app.route('/api/exploit/summary', methods=['GET'])
def get_summary():
    cve = CVE_2025_30065('192.168.1.100', '8080')
    return jsonify(cve.get_summary())


@app.route('/api/exploit/mitigation', methods=['GET'])
def get_mitigation():
    cve = CVE_2025_30065('192.168.1.100', '8080')
    return jsonify(cve.get_mitigation())


if __name__ == '__main__':
    app.run(debug=True)